//+------------------------------------------------------------------+
//|                                               SemaphorPrice2.mq4 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, zznbrm"
                          
//---- indicator settings
#property indicator_chart_window
#property  indicator_buffers 5 
#property  indicator_color1 Lime
#property  indicator_color2 Red
#property  indicator_color3 White
#property  indicator_color4 White
#property  indicator_color5 DimGray
#property  indicator_width1 2
#property  indicator_width2 2
#property  indicator_width3 0
#property  indicator_width4 0
#property  indicator_width5 3

//---- input parameters
extern double edblPips = 20.0;
extern int    eintArrow = 161;

//---- indicator buffers
double gadblHigh[];
double gadblLow[];
double gadblTempHigh[];
double gadblTempLow[];
double gadblExtreme[];

// Program variables
datetime gdtLastHL;
double   gdblTruePoint;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   SetIndexBuffer( 0, gadblHigh );
   SetIndexEmptyValue( 0, 0.0 );
   SetIndexStyle( 0, DRAW_ARROW );
   SetIndexArrow( 0, eintArrow ); 
   SetIndexLabel( 0, "SP-High(" + DoubleToStr( edblPips, 1 ) + ")" );

   SetIndexBuffer( 1, gadblLow );
   SetIndexEmptyValue( 1, 0.0 );
   SetIndexStyle( 1, DRAW_ARROW );
   SetIndexArrow( 1, eintArrow ); 
   SetIndexLabel( 1, "SP-Low(" + DoubleToStr( edblPips, 1 ) + ")" );   
   
   SetIndexBuffer( 2, gadblTempHigh );
   SetIndexEmptyValue( 2, 0.0 );
   SetIndexStyle( 2, DRAW_ARROW );
   SetIndexArrow( 2, eintArrow ); 
   SetIndexLabel( 2, NULL );   
   
   SetIndexBuffer( 3, gadblTempLow );
   SetIndexEmptyValue( 3, 0.0 );
   SetIndexStyle( 3, DRAW_ARROW );
   SetIndexArrow( 3, eintArrow ); 
   SetIndexLabel( 3, NULL );   
   
   SetIndexBuffer( 4, gadblExtreme );
   SetIndexEmptyValue( 4, 0.0 );
   SetIndexStyle( 4, DRAW_SECTION );
   SetIndexLabel( 4, NULL );   
       
   IndicatorDigits( Digits );
     
   //---- name for DataWindow and indicator subwindow label
   IndicatorShortName( "SemaphorPrice(" + DoubleToStr( edblPips, 1 ) + ")" );
   
   gdtLastHL = 0;
   gdblTruePoint = genTruePoint();
   
   return( 0 );
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   return( 0 );
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   if ( !genIsNewBar() )    return( 0 );

   int intLast, intSave;
   
   if ( gdtLastHL == 0 )
   {
      intLast = Bars - 1;
      gdtLastHL = Time[intLast];
      gadblHigh[intLast] = High[intLast];
      gadblLow[intLast] = 0.0;     
   }
   else
   {
      intLast = iBarShift( Symbol(), Period(), MathAbs( gdtLastHL ), true );      
   }
   
   intSave = intLast;
     
   for ( int inx = (intLast-1); inx > 0; inx-- )
   {
      gadblTempHigh[inx] = 0.0;
      gadblTempLow[inx] = 0.0;
      gadblHigh[inx] = 0.0;
      gadblLow[inx] = 0.0;
      
      if ( gdtLastHL > 0 )
      {
         if ( Low[inx] <= ( gadblHigh[intLast] - ( edblPips * gdblTruePoint ) ) )
         {
            if ( High[inx] >= gadblHigh[intLast] )
            {
               gadblTempHigh[intLast] = gadblHigh[intLast];
               gadblHigh[intLast] = 0.0;
               gadblHigh[inx] = High[inx];
            }
            
            intLast = inx;
            gdtLastHL = (-1) * Time[inx];
            gadblLow[inx] = Low[inx];
         }
         else if ( High[inx] >= gadblHigh[intLast] )
         {
            gadblTempHigh[intLast] = gadblHigh[intLast];
            gadblHigh[intLast] = 0.0;
            intLast = inx;     
            gdtLastHL = Time[inx];       
            gadblHigh[inx] = High[inx];
         }
      }   
      else if ( gdtLastHL < 0 )
      {
         if ( High[inx] >= ( gadblLow[intLast] + ( edblPips * gdblTruePoint ) ) )
         {
            if ( Low[inx] <= gadblLow[intLast] )
            {
               gadblTempLow[intLast] = gadblLow[intLast];
               gadblLow[intLast] = 0.0;
               gadblLow[inx] = Low[inx];
            }
            
            intLast = inx;
            gdtLastHL = Time[inx];
            gadblHigh[inx] = High[inx];
         }
         else if ( Low[inx] <= gadblLow[intLast] )
         {
            gadblTempLow[intLast] = gadblLow[intLast];
            gadblLow[intLast] = 0.0;
            intLast = inx;     
            gdtLastHL = (-1) * Time[inx];       
            gadblLow[inx] = Low[inx];
         }
      }   
   }   
   
   for ( inx = (intSave-1); inx > 0; inx-- )
   {
      if ( gadblLow[inx] > 0.0 )        gadblExtreme[inx] = gadblLow[inx];
      else if ( gadblHigh[inx] > 0.0 )  gadblExtreme[inx] = gadblHigh[inx];
      else                              gadblExtreme[inx] = 0.0;
   }
   
   return( 0 );
}

//+------------------------------------------------------------------+
//| genTruePoint()                                                   |
//+------------------------------------------------------------------+
double genTruePoint( string strPair="" )
{
   int intDigits;
   double dblPoint;
   
   if ( strPair == "" || strPair == Symbol() )    intDigits = Digits;
   else                                           intDigits = MarketInfo( strPair, MODE_DIGITS );
      
   // Reset point value for 5 digit brokers
   switch( intDigits )
   {
      case 5:
      case 4:     dblPoint = 0.0001;   break;
      case 3:
      case 2:     dblPoint = 0.01;     break;
      case 1:     dblPoint = 0.1;      break;
      case 0:     dblPoint = 0.0;      break;
      default:    dblPoint = 0.0001;
   }
   
   return( dblPoint );
}

//+------------------------------------------------------------------+
//| genIsNewBar()                                                    |
//+------------------------------------------------------------------+
bool genIsNewBar() 
{
   bool blnReturn = false;
   static datetime dtLastTime = 0;
   
   if ( Time[0] != dtLastTime ) 
   {
      dtLastTime = Time[0];	
      blnReturn = true;
   }
   
   return( blnReturn );
}

